// regular3.cpp : R\[ AvP[Vp̃Gg |Cg̒`
//

#include "stdafx.h"

//Wo͂ƃG[o

XMLStdErr  errStrm;
XMLStdOut  outStrm;

static char*    schemaFile           = 0;
static char*    instanceFile         = 0;

// ---------------------------------------------------------------------------
//  Local helper methods
// ---------------------------------------------------------------------------
static void usage()
{
    outStrm <<  "\nUsage: rv [options] schema instance\n"
            <<  "   This program validates an instance against a regular schema\n\n"
            <<  "   Options:\n"
            <<  "     -?          Show this help\n"
            <<  EndLn;
}

static bool read_instance(DOM_Document& doc, Schema& schm)
{
	//
    //  Create a NonValidatingSAXParser and an InstanceHandler. 
    //

	NonValidatingSAXParser* inst_parser = new NonValidatingSAXParser;
    InstanceHandler*  inst_handler = new InstanceHandler(&schm);
    inst_parser->setDocumentHandler(inst_handler);

	//
    //  Attach an error handler to the parser. The parser will call back to
    //  methods of the ErrorHandler if it discovers errors during the course
    //  of parsing the XML document.
    //

    ErrorHandler *errReporter = new ErrorReporter();
    inst_parser->setErrorHandler(errReporter);


    // Parse the instance file and catch any exceptions that propogate out
    try
    {
        inst_parser->parse(instanceFile);
    }

    catch (const XMLException& toCatch)
    {
        errStrm << "\nFile not found: '" << instanceFile << "'\n"
                << "Exception message is: \n"
                << toCatch.getMessage() << "\n" << EndLn;
        return false;
    }

	//Report the result

	if (inst_handler->isValid())
		cout << "This document is valid." << endl;
	else
		cout << "This document is invalid." << endl;

	// Cleanup
    delete errReporter; 
    delete inst_parser;
    delete inst_handler;

	return true;
}

static bool read_schema(DOM_Document& doc, bool useVDOMParser)
{
	//
    //  Instantiate the appropriate DOM parser. 
    //
	NonValidatingDOMParser* parser;
    if (useVDOMParser == true)
        parser = new ValidatingDOMParser;
    else
        parser = new NonValidatingDOMParser;

    //
    //  Attach an error handler to the parser. The parser will call back to
    //  methods of the ErrorHandler if it discovers errors during the course
    //  of parsing the XML document.
    //
	
    ErrorHandler *errReporter = new ErrorReporter();
    parser->setErrorHandler(errReporter);

    //
    //  Parse the XML file, catching any XML exceptions that might propogate
    //  out of it.
    //
    try
    {
        parser->parse(schemaFile);
    }    

    catch (const XMLException& e)
    {
        errStrm << "An error occured during parsing\n   Message: "
                << e.getMessage() << EndLn;
        return false;
    }
	
    doc = parser->getDocument();

    // Cleanup
    delete parser;
    delete errReporter; 

	return true;
}


int main(int argc, char* argv[])
{
	
    unsigned int parmInd = 0;
	bool     useVDOMParser     = false;

    try
    {
        XMLPlatformUtils::Initialize();
    }

    catch(const XMLException& toCatch)
    {
        errStrm << "Error during XML4C2 Initialization.\n"
                   "  Exception message is \"%s\"\n"
                << toCatch.getMessage() << EndLn;
        return 1;
    }

   // Check command line and extract arguments.
    if (argc < 3)
    {
        usage();
        return 1;
    }

    // Watch for special case help request
    if (strcmp(argv[1], "-?") == 0)
    {
        usage();
        return 0;
    }

	// See if non validating dom parser configuration is requested.
    for (parmInd = 1; parmInd < (unsigned int)argc; parmInd++)
    {
        // Break out on first parm not starting with a dash
        if (argv[parmInd][0] != '-')
            break;

        if (!strcmp(argv[parmInd], "-v")
        ||  !strcmp(argv[parmInd], "-V"))
        {
            useVDOMParser = true;
        }
         else
        {
            usage();
            return 1;
        }
    }


    schemaFile = argv[parmInd];
    instanceFile = argv[parmInd+1];

    DOM_Document doc;
	if (!(read_schema(doc, useVDOMParser)))
		return 1;

	Schema schm(&doc);
	schm.initialize();

	if (!(read_instance(doc, schm)))
		return 1;

    return 0;
}

